Package: plyranges
Type: Package
Title: A fluent interface for manipulating GenomicRanges
Version: 1.32.0
Authors@R: c(
      person("Stuart", "Lee", role = c("aut"), 
             comment = c(ORCID = "0000-0003-1179-8436")),
      person("Michael", "Lawrence", role = c("aut", "ctb")),
      person("Dianne", "Cook", role = c("aut", "ctb")),
      person("Spencer", "Nystrom", role = c("ctb"), 
             comment = c(ORCID = "0000-0003-1000-1579")),
      person("Pierre-Paul", "Axisa", role=c("ctb")),
      person("Michael", "Love", role = c("ctb","cre"),
             email = "michaelisaiahlove@gmail.com")
      )
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
Description: A dplyr-like interface for interacting with the common Bioconductor 
    classes Ranges and GenomicRanges. By providing a grammatical
    and consistent way of manipulating these classes their accessiblity for new
    Bioconductor users is hopefully increased.
URL: https://tidyomics.github.io/plyranges
Depends: R (>= 3.5), BiocGenerics, IRanges (>= 2.12.0), GenomicRanges
        (>= 1.28.4), dplyr
Imports: methods, rlang (>= 0.2.0), magrittr, tidyselect (>= 1.0.0),
        rtracklayer, GenomicAlignments, Seqinfo, Rsamtools, S4Vectors
        (>= 0.23.10), utils
biocViews: Infrastructure, DataRepresentation, WorkflowStep, Coverage
BugReports: https://github.com/tidyomics/plyranges
License: Artistic-2.0
Encoding: UTF-8
ByteCompile: true
Suggests: knitr, BiocStyle, rmarkdown, testthat (>= 2.1.0),
        HelloRanges, HelloRangesData, BSgenome.Hsapiens.UCSC.hg19,
        pasillaBamSubset, covr, ggplot2
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Collate: 'class-AnchoredRanges.R' 'class-Operator.R'
        'class-DeferredGenomicRanges.R' 'class-GroupedRanges.R'
        'dplyr-arrange.R' 'dplyr-filter.R' 'dplyr-groups.R'
        'dplyr-mutate.R' 'dplyr-pull.R' 'dplyr-select.R'
        'dplyr-slice.R' 'dplyr-summarize.R' 'endo-coverage.R'
        'endo-tile.R' 'io-bam.R' 'io-bed.R' 'io-bigwig.R' 'io-gff.R'
        'io-wig.R' 'methods-DeferredGenomicRanges.R'
        'methods-Operator.R' 'plyranges.R' 'ranges-add-distance.R'
        'ranges-anchors.R' 'ranges-arithmetic-flank.R'
        'ranges-arithmetic-setters.R' 'ranges-arithmetic-shift.R'
        'ranges-arithmetic-stretch.R' 'ranges-bind.R' 'ranges-chop.R'
        'ranges-colwise.R' 'ranges-construct.R' 'ranges-disjoin.R'
        'ranges-eval-quo.R' 'ranges-eval.R' 'ranges-expand.R'
        'ranges-genomeinfo.R' 'ranges-join-follow.R'
        'ranges-join-mcols.R' 'ranges-join-nearest.R'
        'ranges-join-precede.R' 'ranges-overlap-count.R'
        'ranges-overlap-filter.R' 'ranges-overlap-find.R'
        'ranges-overlap-groups.R' 'ranges-overlap-joins-intersect.R'
        'ranges-overlap-joins-outer.R' 'ranges-overlap-self-joins.R'
        'ranges-pairs.R' 'ranges-rangewise-setops.R' 'ranges-reduce.R'
        'ranges-setops.R' 'utils-pipe.R'
git_url: https://git.bioconductor.org/packages/plyranges
git_branch: RELEASE_3_23
git_last_commit: 1ace171
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:54:57 UTC; biocbuild
Author: Stuart Lee [aut] (ORCID: <https://orcid.org/0000-0003-1179-8436>),
  Michael Lawrence [aut, ctb],
  Dianne Cook [aut, ctb],
  Spencer Nystrom [ctb] (ORCID: <https://orcid.org/0000-0003-1000-1579>),
  Pierre-Paul Axisa [ctb],
  Michael Love [ctb, cre]
