# Generated by roxygen2: do not edit by hand

S3method(anchor,AnchoredGenomicRanges)
S3method(anchor,AnchoredIntegerRanges)
S3method(anchor_3p,GenomicRanges)
S3method(anchor_5p,GenomicRanges)
S3method(anchor_center,GenomicRanges)
S3method(anchor_center,IntegerRanges)
S3method(anchor_end,GenomicRanges)
S3method(anchor_end,IntegerRanges)
S3method(anchor_start,GenomicRanges)
S3method(anchor_start,IntegerRanges)
S3method(arrange,DelegatingGenomicRanges)
S3method(arrange,DelegatingIntegerRanges)
S3method(arrange,Ranges)
S3method(as_granges,DataFrame)
S3method(as_granges,data.frame)
S3method(as_granges,default)
S3method(as_iranges,DataFrame)
S3method(as_iranges,data.frame)
S3method(as_iranges,default)
S3method(as_ranges,Rle)
S3method(as_ranges,RleList)
S3method(chop_by_gaps,DeferredGenomicRanges)
S3method(chop_by_gaps,GenomicRanges)
S3method(chop_by_introns,DeferredGenomicRanges)
S3method(chop_by_introns,GenomicRanges)
S3method(complement_ranges,GenomicRanges)
S3method(complement_ranges,IntegerRanges)
S3method(complement_ranges_directed,GenomicRanges)
S3method(compute_coverage,default)
S3method(count_overlaps,GenomicRanges)
S3method(count_overlaps,IntegerRanges)
S3method(count_overlaps_directed,GenomicRanges)
S3method(count_overlaps_within,GenomicRanges)
S3method(count_overlaps_within,IntegerRanges)
S3method(count_overlaps_within_directed,GenomicRanges)
S3method(disjoin_ranges,GenomicRanges)
S3method(disjoin_ranges,GroupedGenomicRanges)
S3method(disjoin_ranges,GroupedIntegerRanges)
S3method(disjoin_ranges,IntegerRanges)
S3method(disjoin_ranges_directed,GenomicRanges)
S3method(disjoin_ranges_directed,GroupedGenomicRanges)
S3method(dplyr::pull,Ranges)
S3method(filter,BamFileOperator)
S3method(filter,DeferredGenomicRanges)
S3method(filter,DelegatingGenomicRanges)
S3method(filter,DelegatingIntegerRanges)
S3method(filter,GroupedGenomicRanges)
S3method(filter,GroupedIntegerRanges)
S3method(filter,Ranges)
S3method(filter_by_non_overlaps,GenomicRanges)
S3method(filter_by_non_overlaps,IntegerRanges)
S3method(filter_by_non_overlaps_directed,GenomicRanges)
S3method(filter_by_overlaps,BamFileOperator)
S3method(filter_by_overlaps,DeferredGenomicRanges)
S3method(filter_by_overlaps,GenomicRanges)
S3method(filter_by_overlaps,IntegerRanges)
S3method(filter_by_overlaps_directed,GenomicRanges)
S3method(find_overlaps,GenomicRanges)
S3method(find_overlaps,IntegerRanges)
S3method(find_overlaps_directed,GenomicRanges)
S3method(find_overlaps_within,GenomicRanges)
S3method(find_overlaps_within,IntegerRanges)
S3method(find_overlaps_within_directed,GenomicRanges)
S3method(get_genome_info,BamFileOperator)
S3method(get_genome_info,DeferredGenomicRanges)
S3method(get_genome_info,Seqinfo)
S3method(get_genome_info,default)
S3method(group_by,DeferredGenomicRanges)
S3method(group_by,GenomicRanges)
S3method(group_by,GroupedGenomicRanges)
S3method(group_by,GroupedIntegerRanges)
S3method(group_by,IntegerRanges)
S3method(group_by_overlaps,GenomicRanges)
S3method(group_by_overlaps,IntegerRanges)
S3method(group_data,GroupedGenomicRanges)
S3method(group_data,GroupedIntegerRanges)
S3method(group_indices,GroupedGenomicRanges)
S3method(group_indices,GroupedIntegerRanges)
S3method(group_indices,Ranges)
S3method(group_keys,GroupedGenomicRanges)
S3method(group_keys,GroupedIntegerRanges)
S3method(group_keys,Ranges)
S3method(group_split,GroupedGenomicRanges)
S3method(group_split,GroupedIntegerRanges)
S3method(group_split,Ranges)
S3method(group_vars,GroupedGenomicRanges)
S3method(group_vars,GroupedIntegerRanges)
S3method(group_vars,Ranges)
S3method(groups,GroupedGenomicRanges)
S3method(groups,GroupedIntegerRanges)
S3method(groups,Ranges)
S3method(intersect_ranges,GenomicRanges)
S3method(intersect_ranges,IntegerRanges)
S3method(intersect_ranges_directed,GenomicRanges)
S3method(join_follow,GenomicRanges)
S3method(join_follow,IntegerRanges)
S3method(join_follow_left,GenomicRanges)
S3method(join_follow_left,IntegerRanges)
S3method(join_follow_upstream,GenomicRanges)
S3method(join_nearest_downstream,GenomicRanges)
S3method(join_nearest_upstream,GenomicRanges)
S3method(join_overlap_intersect,GenomicRanges)
S3method(join_overlap_intersect,IntegerRanges)
S3method(join_overlap_intersect_directed,GenomicRanges)
S3method(join_overlap_intersect_within,GenomicRanges)
S3method(join_overlap_intersect_within,IntegerRanges)
S3method(join_overlap_intersect_within_directed,GenomicRanges)
S3method(join_overlap_left,GenomicRanges)
S3method(join_overlap_left,IntegerRanges)
S3method(join_overlap_left_directed,GenomicRanges)
S3method(join_overlap_left_within,GenomicRanges)
S3method(join_overlap_left_within,IntegerRanges)
S3method(join_overlap_left_within_directed,GenomicRanges)
S3method(join_overlap_self,GenomicRanges)
S3method(join_overlap_self,IntegerRanges)
S3method(join_overlap_self_directed,GenomicRanges)
S3method(join_overlap_self_within,GenomicRanges)
S3method(join_overlap_self_within,IntegerRanges)
S3method(join_overlap_self_within_directed,GenomicRanges)
S3method(join_precede,GenomicRanges)
S3method(join_precede,IntegerRanges)
S3method(join_precede_downstream,GenomicRanges)
S3method(join_precede_right,GenomicRanges)
S3method(join_precede_right,IntegerRanges)
S3method(load_genomic_file,BamFileOperator)
S3method(mutate,AnchoredGenomicRanges)
S3method(mutate,AnchoredIntegerRanges)
S3method(mutate,DeferredGenomicRanges)
S3method(mutate,DelegatingGenomicRanges)
S3method(mutate,DelegatingIntegerRanges)
S3method(mutate,GroupedGenomicRanges)
S3method(mutate,GroupedIntegerRanges)
S3method(mutate,Ranges)
S3method(overscope_ranges,DelegatingGenomicRanges)
S3method(overscope_ranges,DelegatingIntegerRanges)
S3method(overscope_ranges,GroupedGenomicRanges)
S3method(overscope_ranges,GroupedIntegerRanges)
S3method(overscope_ranges,Ranges)
S3method(pair_follow,GenomicRanges)
S3method(pair_follow,IntegerRanges)
S3method(pair_nearest,GenomicRanges)
S3method(pair_nearest,IntegerRanges)
S3method(pair_overlaps,GenomicRanges)
S3method(pair_overlaps,IntegerRanges)
S3method(pair_precede,GenomicRanges)
S3method(pair_precede,IntegerRanges)
S3method(pull,DelegatingGenomicRanges)
S3method(pull,DelegatingIntegerRanges)
S3method(pull,GroupedGenomicRanges)
S3method(pull,GroupedIntegerRanges)
S3method(reduce_ranges,GenomicRanges)
S3method(reduce_ranges,GroupedGenomicRanges)
S3method(reduce_ranges,GroupedIntegerRanges)
S3method(reduce_ranges,IntegerRanges)
S3method(reduce_ranges_directed,GenomicRanges)
S3method(reduce_ranges_directed,GroupedGenomicRanges)
S3method(select,BamFileOperator)
S3method(select,DeferredGenomicRanges)
S3method(select,DelegatingGenomicRanges)
S3method(select,DelegatingIntegerRanges)
S3method(select,Ranges)
S3method(set_end,Ranges)
S3method(set_seqnames,GenomicRanges)
S3method(set_start,Ranges)
S3method(set_strand,GenomicRanges)
S3method(set_width,AnchoredGenomicRanges)
S3method(set_width,AnchoredIntegerRanges)
S3method(set_width,Ranges)
S3method(setdiff_ranges,GenomicRanges)
S3method(setdiff_ranges,IntegerRanges)
S3method(setdiff_ranges_directed,GenomicRanges)
S3method(slice,GroupedGenomicRanges)
S3method(slice,GroupedIntegerRanges)
S3method(slice,Ranges)
S3method(slice_head,GroupedGenomicRanges)
S3method(slice_head,GroupedIntegerRanges)
S3method(slice_head,Ranges)
S3method(slice_max,GroupedGenomicRanges)
S3method(slice_max,GroupedIntegerRanges)
S3method(slice_max,Ranges)
S3method(slice_min,GroupedGenomicRanges)
S3method(slice_min,GroupedIntegerRanges)
S3method(slice_min,Ranges)
S3method(slice_sample,GroupedGenomicRanges)
S3method(slice_sample,GroupedIntegerRanges)
S3method(slice_sample,Ranges)
S3method(slice_tail,GroupedGenomicRanges)
S3method(slice_tail,GroupedIntegerRanges)
S3method(slice_tail,Ranges)
S3method(slide_ranges,Ranges)
S3method(stretch,AnchoredGenomicRanges)
S3method(stretch,AnchoredIntegerRanges)
S3method(stretch,Ranges)
S3method(summarise,DeferredGenomicRanges)
S3method(summarise,DelegatingGenomicRanges)
S3method(summarise,GroupedGenomicRanges)
S3method(summarise,GroupedIntegerRanges)
S3method(summarise,Ranges)
S3method(tbl_vars,GenomicRanges)
S3method(tbl_vars,Ranges)
S3method(tile_ranges,Ranges)
S3method(unanchor,AnchoredGenomicRanges)
S3method(unanchor,AnchoredIntegerRanges)
S3method(ungroup,GroupedGenomicRanges)
S3method(ungroup,GroupedIntegerRanges)
S3method(ungroup,Ranges)
S3method(union_ranges,GenomicRanges)
S3method(union_ranges,IntegerRanges)
S3method(union_ranges_directed,GenomicRanges)
export("%>%")
export("%intersect%")
export("%setdiff%")
export("%union%")
export(add_nearest_distance)
export(add_nearest_distance_downstream)
export(add_nearest_distance_left)
export(add_nearest_distance_right)
export(add_nearest_distance_upstream)
export(anchor)
export(anchor_3p)
export(anchor_5p)
export(anchor_center)
export(anchor_centre)
export(anchor_end)
export(anchor_start)
export(as_granges)
export(as_iranges)
export(as_ranges)
export(between)
export(bind_ranges)
export(chop_by_gaps)
export(chop_by_introns)
export(complement_ranges)
export(complement_ranges_directed)
export(compute_coverage)
export(count_overlaps)
export(count_overlaps_directed)
export(count_overlaps_within)
export(count_overlaps_within_directed)
export(disjoin_ranges)
export(disjoin_ranges_directed)
export(expand_ranges)
export(filter_by_non_overlaps)
export(filter_by_non_overlaps_directed)
export(filter_by_overlaps)
export(filter_by_overlaps_directed)
export(find_overlaps)
export(find_overlaps_directed)
export(find_overlaps_within)
export(find_overlaps_within_directed)
export(flank_downstream)
export(flank_left)
export(flank_right)
export(flank_upstream)
export(genome_info)
export(get_genome_info)
export(group_by_overlaps)
export(id_to_column)
export(intersect_ranges)
export(intersect_ranges_directed)
export(interweave)
export(join_follow)
export(join_follow_left)
export(join_follow_upstream)
export(join_mcols_inner)
export(join_mcols_left)
export(join_nearest)
export(join_nearest_downstream)
export(join_nearest_left)
export(join_nearest_right)
export(join_nearest_upstream)
export(join_overlap_inner)
export(join_overlap_inner_directed)
export(join_overlap_inner_within)
export(join_overlap_inner_within_directed)
export(join_overlap_intersect)
export(join_overlap_intersect_directed)
export(join_overlap_intersect_within)
export(join_overlap_intersect_within_directed)
export(join_overlap_left)
export(join_overlap_left_directed)
export(join_overlap_left_within)
export(join_overlap_left_within_directed)
export(join_overlap_self)
export(join_overlap_self_directed)
export(join_overlap_self_within)
export(join_overlap_self_within_directed)
export(join_precede)
export(join_precede_downstream)
export(join_precede_right)
export(n)
export(n_distinct)
export(names_to_column)
export(overscope_ranges)
export(pair_follow)
export(pair_nearest)
export(pair_overlaps)
export(pair_precede)
export(read_bam)
export(read_bed)
export(read_bed_graph)
export(read_bigwig)
export(read_gff)
export(read_gff1)
export(read_gff2)
export(read_gff3)
export(read_narrowpeaks)
export(read_wig)
export(reduce_ranges)
export(reduce_ranges_directed)
export(remove_names)
export(set_end)
export(set_genome_info)
export(set_seqnames)
export(set_start)
export(set_strand)
export(set_width)
export(setdiff_ranges)
export(setdiff_ranges_directed)
export(shift_downstream)
export(shift_left)
export(shift_right)
export(shift_upstream)
export(slide_ranges)
export(span)
export(stretch)
export(tile_ranges)
export(unanchor)
export(union_ranges)
export(union_ranges_directed)
export(write_bed)
export(write_bed_graph)
export(write_bigwig)
export(write_gff)
export(write_gff1)
export(write_gff2)
export(write_gff3)
export(write_narrowpeaks)
export(write_wig)
exportClasses(BamFileOperator)
exportClasses(DeferredGenomicRanges)
exportClasses(FileOperator)
exportClasses(GroupedGenomicRanges)
exportClasses(GroupedIntegerRanges)
importFrom(BiocGenerics,"%in%")
importFrom(BiocGenerics,"end<-")
importFrom(BiocGenerics,"score<-")
importFrom(BiocGenerics,"start<-")
importFrom(BiocGenerics,"strand<-")
importFrom(BiocGenerics,"width<-")
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,rep.int)
importFrom(BiocGenerics,score)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,strand)
importFrom(BiocGenerics,unique)
importFrom(BiocGenerics,unlist)
importFrom(BiocGenerics,which)
importFrom(BiocGenerics,width)
importFrom(GenomicAlignments,extractAlignmentRangesOnReference)
importFrom(GenomicAlignments,readGAlignmentPairs)
importFrom(GenomicAlignments,readGAlignments)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,distanceToNearest)
importFrom(GenomicRanges,granges)
importFrom(IRanges,IRanges)
importFrom(IRanges,as.env)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,coverage)
importFrom(IRanges,disjoin)
importFrom(IRanges,distanceToNearest)
importFrom(IRanges,extractList)
importFrom(IRanges,findOverlapPairs)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,flank)
importFrom(IRanges,follow)
importFrom(IRanges,mid)
importFrom(IRanges,nearest)
importFrom(IRanges,pgap)
importFrom(IRanges,pintersect)
importFrom(IRanges,precede)
importFrom(IRanges,psetdiff)
importFrom(IRanges,punion)
importFrom(IRanges,ranges)
importFrom(IRanges,reduce)
importFrom(IRanges,resize)
importFrom(IRanges,shift)
importFrom(IRanges,slidingWindows)
importFrom(IRanges,subsetByOverlaps)
importFrom(IRanges,tile)
importFrom(Rsamtools,"bamFlag<-")
importFrom(Rsamtools,"bamMapqFilter<-")
importFrom(Rsamtools,"bamTag<-")
importFrom(Rsamtools,"bamWhat<-")
importFrom(Rsamtools,"bamWhich<-")
importFrom(Rsamtools,BamFile)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,bamFlag)
importFrom(Rsamtools,bamWhich)
importFrom(Rsamtools,scanBamFlag)
importFrom(Rsamtools,scanBamWhat)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Pairs)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,cbind)
importFrom(S4Vectors,decode)
importFrom(S4Vectors,elementNROWS)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,expand)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,merge)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,parallelVectorNames)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,queryLength)
importFrom(S4Vectors,rbind)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,zipup)
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,"seqnames<-")
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,isCircular)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_data)
importFrom(dplyr,group_indices)
importFrom(dplyr,group_keys)
importFrom(dplyr,group_split)
importFrom(dplyr,group_vars)
importFrom(dplyr,groups)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,slice_sample)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,tbl_vars)
importFrom(dplyr,ungroup)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,getGeneric)
importFrom(methods,getGenerics)
importFrom(methods,hasMethod)
importFrom(methods,initialize)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,selectMethod)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,setValidity)
importFrom(methods,show)
importFrom(rlang,"!!!")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,dots_values)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,env_bind)
importFrom(rlang,env_get)
importFrom(rlang,env_parent)
importFrom(rlang,eval_tidy)
importFrom(rlang,have_name)
importFrom(rlang,new_data_mask)
importFrom(rlang,quo_is_missing)
importFrom(rlang,quo_name)
importFrom(rlang,quos)
importFrom(rlang,syms)
importFrom(rtracklayer,BigWigFile)
importFrom(rtracklayer,BigWigSelection)
importFrom(rtracklayer,export.bed)
importFrom(rtracklayer,export.bedGraph)
importFrom(rtracklayer,export.bw)
importFrom(rtracklayer,export.gff)
importFrom(rtracklayer,export.gff1)
importFrom(rtracklayer,export.gff2)
importFrom(rtracklayer,export.gff3)
importFrom(rtracklayer,export.wig)
importFrom(rtracklayer,import.bed)
importFrom(rtracklayer,import.bedGraph)
importFrom(rtracklayer,import.bw)
importFrom(rtracklayer,import.gff)
importFrom(rtracklayer,import.gff1)
importFrom(rtracklayer,import.gff2)
importFrom(rtracklayer,import.gff3)
importFrom(rtracklayer,import.wig)
importFrom(tidyselect,eval_select)
importFrom(utils,relist)
